import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class CalendarReader
{
	private String baseURL;
	private URL url;
	private String documentAsText;
	private Document document;
	private boolean loadedXML;
	private boolean loadedText;
	private String calendarTitle;
	
	public CalendarReader(String location) throws MalformedURLException
	{
		// store the location as the calendar's Base URL
		baseURL = location; 
	
		// append the necessary search parameters
		location = location + "?max-results=999999";
		
		// save the url location
		url = new URL(location);
		
		// initialize the document
		documentAsText = new String();
		
		// make that the document has not yet been loaded
		loadedText = false;
		loadedXML = false;
	}
	
	// returns the original URL entered by the User
	public String getURL()
	{
		return baseURL;
	}
	
	// this method will load the document into a string to be used later in the old
	// method of text based parsing to extract the data
	public void loadText() throws IOException
	{
		// create the reader that will read from the file at location in url
		BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));

		// line stores the next line of the file
		String line;

		// while there is data to read
		while ((line = in.readLine()) != null)
		{
			// save the data into document
			documentAsText = documentAsText + line;
		}

		// close the connection
		in.close();
		
		// mark that the data was successfully loaded
		loadedText = true;
	}
	
	// this function will load the document objects to be used for XML parsing
	public void loadXML() throws Exception
	{
		// configure the Document builder
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		
		// parse the XML document at the designated URL
		
		//TODO offline testing
		//document = db.parse(new File("c:/documents and settings/isaac/desktop/basic.xhtml"));
		document = db.parse(url.openStream());
		
		// mark that the data was successfully loaded
		loadedXML = true;
	}
	
	// this method returns the text of the document
	public String getText() throws DataNotLoaded
	{
		if(!loadedText)
		{
			throw new DataNotLoaded();
		}
		// return the document
		return documentAsText;
	}
	
	public String getTitle()
	{
		return calendarTitle;
	}
	
	// this method uses the new method of XML parsing to extract data and create
	// the returned list of Invoice Entries it also sets the Calendar Title
	public List<InvoiceEntry> getInvoicesFromXML() throws DataNotLoaded
	{
		// ensure the XML document was loaded and therefore ready for reading
		if(!getLoadedXML())
		{
			throw new DataNotLoaded();
		}
		
		// create the array of invoice entries to be returned at the end
		ArrayList<InvoiceEntry> invoice = new ArrayList<InvoiceEntry>();
		
		//get the root element
		Element docEle = document.getDocumentElement();

		// get a nodelist of  elements in the XML document
		// one element is defined by that content between "entry" tags
		NodeList nl = docEle.getElementsByTagName("entry");
		
		// if there are Nodes in the NodeList
		if(nl != null && nl.getLength() > 0) 
		{
			// iterate through the list of Nodes
			for(int i = 0 ; i < nl.getLength();i++) 
			{
				// get the invoice entry element
				Element el = (Element)nl.item(i);

				try 
				{
					// get the InvoiceEntry object from the extracted Element
					InvoiceEntry ie = getInvoiceEntry(el);
					
					// add it to list
					invoice.add(ie);
				}
				catch (DateFormatError e) 
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		// if there are no Nodes in the List, or the list is empty
		else
		{
			// do nothing, return an empty list
		}
		
		// at this point all the invoices are unsorted, but it should remain like this for now
		
		String title = getTextValue(docEle, "title");
		System.out.println(title);
		calendarTitle = title;
		
		// return the final sorted list
		return invoice;
	}
	
	// this method is called by "getInvoicesFromXML" and is designed to take
	// a parsed XML "entry" element and convert the content into a new Invoice Entry
	private InvoiceEntry getInvoiceEntry(Element el) throws DateFormatError
	{
		// extract the entry's title and content (content stores additional information)
		String title = getTextValue(el,"title");
		String content = getTextValue(el, "content");
		
		// Strip the content of XML encoding (IE: &quot; instead of ")
		StringBuffer sb = new StringBuffer(content);
		
		// the position of the next convertible string of characters
		int pos;
		
		// while there remains objects to convert into "s,
		while((pos = sb.indexOf("&quot;")) > -1)
		{
			// convert them into "s
			sb.replace(pos, pos+6, "\"");
		}
		
		// while there remains objects to convert into 's,
		while((pos = sb.indexOf("&#39;")) > -1)
		{
			// convert them into 's
			sb.replace(pos, pos+5, "'");
		}
		
		// revaluate content to equal the newly formatted string from the string buffer
		content = sb.toString();
		
		// parse the content string for the various Invoice Entry parameters
		// these include the Date, Location, and Description
		String dateText = parseContent(content, "When: ");
		String location = parseContent(content, "Where: ");
		String description = parseContent(content, "Event Description: ");
		
		// create the invoice entry with the harvested data
		InvoiceEntry retVal = new InvoiceEntry(title, description, location, dateText, this);
		
		// return the resulting InvoiceEntry
		return retVal;
	}
	
	// this function takes the element object and a tagname and extracts the string information stored therein
	private String getTextValue(Element ele, String tagName) 
	{
		// the return value containing the text between the designated tags
		String textVal = null;
		
		// the list of Nodes within the designated tags
		NodeList nl = ele.getElementsByTagName(tagName);
		
		// if the list of nodes is populated,
		if(nl != null && nl.getLength() > 0) 
		{
			// get the element at the node
			Element el = (Element)nl.item(0);
			
			// get the content inside the tags
			textVal = el.getFirstChild().getNodeValue();
		}

		// return the content
		return textVal;
	}
	
	// this function returns whether or not the XML Document was successfully loaded
	public boolean getLoadedXML()
	{
		return loadedXML;
	}
	
	// this function takes the content string and looks for the data with the title
	// matching the passed parameter.
	private String parseContent(String content, String elementTitle)
	{
		// retVal will store the data extracted after parsing the content string for the 
		// designated elementTitle
		String retVal = null;
		
		// grab the content information
		String [] contentDetails = content.split("<br />");
		
		// The order of the content details goes, "When:", "\n\n", "Where:", "Event Status:", "Event Description:",
		for(int i=0; i<contentDetails.length; i++)
		{
			// if this element contains the designated element title
			// we first remove all the newline characters before saving it
			if(contentDetails[i].contains(elementTitle))
			{
				// while the string contains a newline character...
				while(contentDetails[i].contains("\n"))
				{
					// remove the next newline character
					contentDetails[i] = contentDetails[i].substring(0, contentDetails[i].indexOf('\n')) + contentDetails[i].substring(contentDetails[i].indexOf('\n')+1);
				}
				
				// save the content into the return value after concatenating the element title;
				retVal = contentDetails[i].substring(contentDetails[i].indexOf(": ")+2);
				
				// exit the loop
				break;
			}
		}
		
		// return the data
		return retVal;
	}

	// used to determine if two calendars are the same
	public boolean equals(CalendarReader other) 
	{
		boolean retVal = false;
		
		// if the other reader is not null
		if(other != null)
		{
			// retVal stores whether two calendars are the same object
			retVal = (this == other);
		}
		
		// return the result
		return retVal;
	}
}
